#include <iostream>
#include <iomanip>
#include <vector>
#include <sstream>
#include <netinet/in.h>
#include <cstring>
#include <cstdlib>

#include "TApplication.h"
#include "TCanvas.h"
#include "TH1F.h"
#include "TThread.h"
#include "TStyle.h"

#include "event.h"


int decode_data(char *buf, std::vector<int> &data)
{
	struct event_header *eheader = reinterpret_cast<struct event_header *>(buf);
	unsigned int *datablock = reinterpret_cast<unsigned int *>(buf + sizeof(struct event_header));
	//unsigned int magic = ntohl(eheader->magic);
	unsigned int magic = eheader->magic;
	int ndata;
	data.clear();
	data.resize(2);

	if (magic == EV_MAGIC) {
		data[0] = static_cast<int>(datablock[0]);
		data[1] = static_cast<int>(datablock[1]);
		ndata = 2;
	} else {
		data[0] = -1;
		data[1] = -1;
		ndata = 0;
		std::cout << "MAGIC ERROR : " << std::hex << magic
			<< std::dec << std::endl;
	}

	return ndata;
}

#define NTDC 2
static TH1F *htdc[NTDC];

int hist_init(){
//	for (int i = 0 ; i < NTDC ; i++) {
		int i=0;
		std::ostringstream ossname;
		std::ostringstream osstitle;
		ossname << "TDC" << i;
		osstitle << "TDC-" << i;
		htdc[i] = new TH1F(ossname.str().c_str(), osstitle.str().c_str(),
			10000,-10000,0);
		htdc[i]->SetTitle("TEMP. Monitor");
		htdc[i]->SetXTitle("Time [s]");
		htdc[i]->SetYTitle("TEMP [Deg. Cel.]");
		htdc[i]->SetLineColor(2);
		htdc[i]->SetFillColor(2);
		htdc[i]->GetYaxis()->SetRangeUser(0,40);
	return 0;
}

int hist_fill(std::vector<int> &vtdc){
	htdc[0]->SetBinContent(9999,vtdc[0]);
	for(int i=0;i<10000;i++){
		htdc[0]->SetBinContent(i,
			htdc[0]->GetBinContent(i+1));
	}
	return 0;
}

static int no_exit = 0;

void reader(void *arg = NULL){
	static char cbuf[1024];
	struct event_header *eheader = reinterpret_cast<struct event_header *>(cbuf);
	unsigned int *datablock = reinterpret_cast<unsigned int *>(cbuf + sizeof(struct event_header));
	std::vector<int> data;
	int event_number = 0, loop_num=0;
	while(1){
		std::cin.read(cbuf, sizeof(struct event_header));
		if (std::cin.eof()) break;
		int rsize = (eheader->size - (sizeof(struct event_header)/sizeof(unsigned int)))
			* sizeof(unsigned int);
		std::cin.read(reinterpret_cast<char *>(datablock), rsize);
		if (std::cin.eof()) break;
		if (decode_data(cbuf, data) > 0) {
			hist_fill(data);
		}
		else{
			std::cerr << "Invalid Header "
				<< static_cast<unsigned int>(cbuf[0]) << std::endl;
			break;
		}
//		if ((event_number+1)%1==0)
//			std::cout << "Event : " << event_number+1 << std::endl;
		if ((event_number+1)%10000==0){
			hist_init();
			loop_num++;
			}
		event_number++;
	}
	if (! no_exit) exit(0);
}


int ta_argc = 1;
char *ta_argv[] = {"App"};
static TApplication *app = new TApplication("App", &ta_argc, ta_argv);
static TCanvas *c1 = new TCanvas("c1", "Online Display");

void updater(void *arg = NULL){
	while(1){
		TThread::Sleep(1, 0);
		c1->Modified();
		c1->Update();
	}
}

int main(int argc, char* argv[]){
	for(int i = 1 ; i < argc ; i++) {
		if(strncmp(argv[i], "--no-exit", 9) == 0)
			no_exit = 1;
	}
	gStyle->SetOptStat(0);

	hist_init();
	htdc[0]->Draw();
	c1->Update();
	c1->Flush();
	TThread threader("Reader", &reader, NULL);
	TThread thupdater("Updater", &updater, NULL);
	threader.Run();
	thupdater.Run();
	app->Run();

	return 0;
}
